% File src/library/base/man/Random-user.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2009 R Core Team
% Distributed under GPL 2 or later

\name{Random.user}
\title{User-supplied Random Number Generation}
\alias{Random.user}
\description{
関数 \code{\link{RNGkind}} を使いユーザがコーディングした一様と正規乱数発生器
が使用可能になる．詳細をここで紹介する．
}
\details{
ユーザ固有の一様 RNG は動的に読み込まれたコンパイル済みコードの
エントリポイントから呼び出される．ユーザはエントリポイント 
  \code{user_unif_rand} を提供する必要がある．これは引数を待たず倍精度実数への
ポインタを返す．以下の例は一般的なパターンを示すであろう．

オプションでユーザは \code{user_unif_init} へのエントリポイントを
与えることができる．これは \code{\link{RNGkind}} (または \code{set.seed}) が
呼び出された時 \code{unsigned int} 引数とともに呼び出され，
ユーザの RNG コードの初期化を可能にすることを意図している．
引数は \sQuote{乱数種} を設定するために使われることを意図している．
これは \code{set.seed} への引数であるか，もし  \code{\link{RNGkind}} が
呼び出されるときは本質的な乱数種である．

もしこれらの関数だけが提供されると， \code{.Random.seed} 中には
生成器の状態の情報は記録されない．オプションで
関数 \code{user_unif_nseed} と \code{user_unif_seedloc} を
提供できる．これは引数なしで呼び出され乱数種の数と乱数種の整数(特に \samp{Int32})配列
へのポインタを返すべきである．
そうすると \code{GetRNGstate} と \code{PutRNGstate} への呼び出しは
この配列を \code{.Random.seed} へ/からコピーする．

ユーザ固有のの正規 RNG は単一のエントリポイント 
 \code{user_norm_rand} により指定され，これは引数を持たず
 倍精度実数へのポインタをケアす．
}
\section{警告}{コンパイルされたすべてのコードと同様に，これらの関数に
ミスがあると \R はクラッシュするかもしれない．タイプの検査のため
 \file{R_ext/Random.h} をインクルードすること．
}
\examples{\dontrun{
##  Marsaglia の乗算合同法 PRNG
#include <R_ext/Random.h>

static Int32 seed;
static double res;
static int nseed = 1;

double * user_unif_rand()
{
    seed = 69069 * seed + 1;
    res = seed * 2.32830643653869e-10;
    return &res;
}

void  user_unif_init(Int32 seed_in) { seed = seed_in; }
int * user_unif_nseed() { return &nseed; }
int * user_unif_seedloc() { return (int *) &seed; }

/*  ratio-of-uniforms for normal  */
#include <math.h>
static double x;

double * user_norm_rand()
{
    double u, v, z;
    do {
        u = unif_rand();
        v = 0.857764 * (2. * unif_rand() - 1);
        x = v/u; z = 0.25 * x * x;
        if (z < 1. - u) break;
        if (z > 0.259/u + 0.35) continue;
    } while (z > -log(u));
    return &x;
}

## Unix 下で使う:
R CMD SHLIB urand.c
R
> dyn.load("urand.so")
> RNGkind("user")
> runif(10)
> .Random.seed
> RNGkind(, "user")
> rnorm(10)
> RNGkind()
[1] "user-supplied" "user-supplied"
}}
\keyword{distribution}
\keyword{sysdata}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/Random-user.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2009 R Core Team
%% Distributed under GPL 2 or later
%
%\name{Random.user}
%\title{User-supplied Random Number Generation}
%\alias{Random.user}
%\description{
%  Function \code{\link{RNGkind}} allows user-coded uniform and
%  normal random number generators to be supplied.  The details are given
%  here.
%}
%\details{
%  A user-specified uniform RNG is called from entry points in
%  dynamically-loaded compiled code.  The user must supply the entry point
%  \code{user_unif_rand}, which takes no arguments and returns a
%  \emph{pointer to} a double. The example below will show the general
%  pattern.
%
%  Optionally, the user can supply the entry point \code{user_unif_init},
%  which is called with an \code{unsigned int} argument when
%  \code{\link{RNGkind}} (or \code{set.seed}) is called, and is intended
%  to be used to initialize the user's RNG code.  The argument is intended
%  to be used to set the \sQuote{seeds}; it is the \code{seed} argument to
%  \code{set.seed} or an essentially random seed if \code{\link{RNGkind}}
%  is called.
%
%  If only these functions are supplied, no information about the
%  generator's state is recorded in \code{.Random.seed}.  Optionally,
%  functions \code{user_unif_nseed} and \code{user_unif_seedloc} can be
%  supplied which are called with no arguments and should return pointers
%  to the number of seeds and to an integer (specifically, \samp{Int32})
%  array of seeds.  Calls to \code{GetRNGstate} and \code{PutRNGstate}
%  will then copy this array to and from \code{.Random.seed}.
%
%  A user-specified normal RNG is specified by a single entry point
%  \code{user_norm_rand}, which takes no arguments and returns a
%  \emph{pointer to} a double.
%}
%\section{Warning}{As with all compiled code, mis-specifying these
%  functions can crash \R.  Do include the \file{R_ext/Random.h}
%  header file for type checking.
%}
%\examples{\dontrun{
%##  Marsaglia's congruential PRNG
%#include <R_ext/Random.h>
%
%static Int32 seed;
%static double res;
%static int nseed = 1;
%
%double * user_unif_rand()
%{
%    seed = 69069 * seed + 1;
%    res = seed * 2.32830643653869e-10;
%    return &res;
%}
%
%void  user_unif_init(Int32 seed_in) { seed = seed_in; }
%int * user_unif_nseed() { return &nseed; }
%int * user_unif_seedloc() { return (int *) &seed; }
%
%/*  ratio-of-uniforms for normal  */
%#include <math.h>
%static double x;
%
%double * user_norm_rand()
%{
%    double u, v, z;
%    do {
%        u = unif_rand();
%        v = 0.857764 * (2. * unif_rand() - 1);
%        x = v/u; z = 0.25 * x * x;
%        if (z < 1. - u) break;
%        if (z > 0.259/u + 0.35) continue;
%    } while (z > -log(u));
%    return &x;
%}
%
%## Use under Unix:
%R CMD SHLIB urand.c
%R
%> dyn.load("urand.so")
%> RNGkind("user")
%> runif(10)
%> .Random.seed
%> RNGkind(, "user")
%> rnorm(10)
%> RNGkind()
%[1] "user-supplied" "user-supplied"
%}}
%\keyword{distribution}
%\keyword{sysdata}
%
\encoding{UTF8}
