% File src/library/base/man/lengths.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2015 R Core Team
% Distributed under GPL 2 or later

\name{lengths}
\alias{lengths}
\title{Lengths of List or Vector Elements}
\encoding{UTF-8}
\description{
  リストの各要素や原子的ベクトル(\code{\link{is.atomic}})の長さを
  整数か数値ベクトルとして返す．
}
\usage{
lengths(x, use.names = TRUE)
}
\arguments{
  \item{x}{\code{\link{list}}，\code{\link{expression}} のような
    リスト風オブジェクト，又は原子的ベクトル(それに対しては結果は自明)．}
  \item{use.names}{結果が \code{x} から名前を引き継ぐかどうかを
    指示する論理値．}
}
\details{
  この関数は \code{x} をループし，その要素の長さを含む一貫したベクトルを返す．
  実効的にはすべての \code{i} に対する 
  \code{length(x[[i]])} が呼びだされるので，
  \code{length} に関するすべてのメソッドが考慮される．
}
\note{
  \code{lengths(x)} の存在理由の一つは，\code{sapply(x, length)} や
  類似の \code{\link{length}} への \code{*apply} 呼び出しのより
  効率的なバージョンとして使うことにある．
  これが \code{lengths(x)} が自明である \code{x} が原子的ベクトルのケース
  を含む理由である．
}
\value{
  長さ \code{length(x)} の非負整数．
  例外は \eqn{2^{31}-1}{2^31 - 1} を超える要素を持つ長さで，
  その時は倍精度実数を返す．
  \code{use.names} が真なら名前は，もしあれば，\code{x} に関する名前から
  取られる．
}
\seealso{
  任意の \R オブジェクトの長さを得るには \code{\link{length}}．
}
\examples{
require(stats)
## 月別に要約
l <- split(airquality$Ozone, airquality$Month)
avgOz <- lapply(l, mean, na.rm=TRUE)
## 結果をマージ
airquality$avgOz <- rep(unlist(avgOz, use.names=FALSE), lengths(l))
## しかしこの方が安全でわかりやすいが，遅くなる可能性
airquality$avgOz <- unsplit(avgOz, airquality$Month)

## 常に真になるべき，長さが32ビットに収まらない場合を除く
stopifnot(identical(lengths(l), vapply(l, length, integer(1L))))

## 空のリストは問題ではない
x <- list()
stopifnot(identical(lengths(x), integer()))

## "リスト風" 表現式も同様：
lengths(expression(u, v, 1+ 0:9))

## そして length メソッドを選択適用すべき
f <- c(rep(1, 3), rep(2, 6), 3)
dates <- split(as.POSIXlt(Sys.time() + 1:10), f)
stopifnot(identical(lengths(dates), vapply(dates, length, integer(1L))))
}
\keyword{attribute}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/base/man/lengths.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2015 R Core Team
%% Distributed under GPL 2 or later
%
%\name{lengths}
%\alias{lengths}
%\title{Lengths of List or Vector Elements}
%\encoding{UTF-8}
%\description{
%  Get the length of each element of a \code{\link{list}} or atomic
%  vector (\code{\link{is.atomic}}) as an integer or numeric vector.
%}
%\usage{
%lengths(x, use.names = TRUE)
%}
%\arguments{
%  \item{x}{a \code{\link{list}}, list-like such as an
%    \code{\link{expression}} or an atomic vector (for which the result
%    is trivial).}
%  \item{use.names}{logical indicating if the result should inherit the
%    \code{\link{names}} from \code{x}.}
%}
%\details{
%  This function loops over \code{x} and returns a compatible vector
%  containing the length of each element in \code{x}.  Effectively,
%  \code{length(x[[i]])} is called for all \code{i}, so any methods on
%  \code{length} are considered.
%}
%\note{
%  One raison d'\enc{être}{etre} of \code{lengths(x)} is its use as a
%  more efficient version of \code{sapply(x, length)} and similar
%  \code{*apply} calls to \code{\link{length}}.  This is the reason why
%  \code{x} may be an atomic vector, even though \code{lengths(x)} is
%  trivial in that case.
%}
%\value{
%  A non-negative \code{\link{integer}} of length \code{length(x)},
%  except when any element has a length of more than
%  \eqn{2^{31}-1}{2^31 - 1} elements, when it returns a double vector.
%  When \code{use.names} is true, the names are taken from the names on
%  \code{x}, if any.
%}
%\seealso{
%  \code{\link{length}} for getting the length of any \R object.
%}
%\examples{
%require(stats)
%## summarize by month
%l <- split(airquality$Ozone, airquality$Month)
%avgOz <- lapply(l, mean, na.rm=TRUE)
%## merge result
%airquality$avgOz <- rep(unlist(avgOz, use.names=FALSE), lengths(l))
%## but this is safer and cleaner, but can be slower
%airquality$avgOz <- unsplit(avgOz, airquality$Month)
%
%## should always be true, except when a length does not fit in 32 bits
%stopifnot(identical(lengths(l), vapply(l, length, integer(1L))))
%
%## empty lists are not a problem
%x <- list()
%stopifnot(identical(lengths(x), integer()))
%
%## nor are "list-like" expressions:
%lengths(expression(u, v, 1+ 0:9))
%
%## and we should dispatch to length methods
%f <- c(rep(1, 3), rep(2, 6), 3)
%dates <- split(as.POSIXlt(Sys.time() + 1:10), f)
%stopifnot(identical(lengths(dates), vapply(dates, length, integer(1L))))
%}
%\keyword{attribute}
%
