% File src/library/graphics/man/layout.Rd
% Part of the R package, https://www.R-project.org
% Copyright 1995-2012 R Core Team
% Distributed under GPL 2 or later

\name{layout}
\alias{layout}
\alias{layout.show}
\alias{lcm}
\title{Specifying Complex Plot Arrangements}
\author{Paul R. Murrell}
\description{
  \code{layout} はデバイスを行列 \code{mat} 中にあるだけの
  列と行に分割し，それぞれの引数中で指定された
  列幅と行高を持つ．
}
\usage{
layout(mat, widths = rep.int(1, ncol(mat)),
       heights = rep.int(1, nrow(mat)), respect = FALSE)

layout.show(n = 1)
lcm(x)
}
\arguments{
  \item{mat}{出力デバイス上の次の \eqn{N} 個の位置を指定する行列．
    行列中の各値は \code{0} か正整数で無ければならない．
    もし \eqn{N} が行列中の最大正整数ならば，整数
    \eqn{\{1, \dots, N-1\}}{{1, \dots, N-1}} は
    少なくとも一回行列中にも登場しなければならない．}
  \item{widths}{デバイス上の列の幅に対するベクトル．
    相対的幅が数値で指定される．絶対的な幅(センチメートル単位)は
    \code{lcm()} 関数で指定される(例を見よ)．}
  \item{heights}{デバイス上の行の高さに対する値のベクトル．
    相対的又は絶対的な高さを指定できる，上の \code{widths} を見よ．}
  \item{respect}{論理値か行列オブジェクト．もし後者なら
    それは \code{mat} と同じ次元を持たねばならず，
    行列中の各値は \code{0} か \code{1} で無ければならない．}
  \item{n}{プロットする図の数．}
  \item{x}{センチメートルの数と解釈される次元．}
}
\details{
    図 \eqn{i} はこれらの行と列の部分集合からなる領域に配置され，
    \eqn{i} が \code{mat} 中に現れる行と列に置かれる．

    \code{respect} 引数はデバイス上列幅の単位が
    行幅の単位と同じ物理的尺度を持つかどうかを制御する．

    レイアウト中の行と列の数には制限(現在 200)があり，
    セルの総数に対してもやはり制限(10007)がある．

    \code{layout.show(n)} は現在のレイアウト(の一部)にプロットする，
    つまり次の \code{n} 図のアウトラインである．

    \code{lcm} は \code{layout()} の
    \code{widths} と \code{heights} 引数に対する絶対次元の指定
    をするインタフェイスとして使われる自明な関数である．
}
\value{
  \code{layout} は図の数 \eqn{N} を返す，上を見よ．
}
\references{
  Murrell, P. R. (1999) Layouts: A mechanism for arranging plots on a page.
  \emph{Journal of Computational and Graphical Statistics},
  \bold{8}, 121--134.

  Chapter 5 of Paul Murrell's Ph.D. thesis.

  Murrell, P. (2005) \emph{R Graphics}. Chapman & Hall/CRC Press.
}
\seealso{
  引数 \code{mfrow}, \code{mfcol} 又は \code{mfg}
  を持つ \code{\link{par}}．
}
\section{警告}{
  これらの関数はデバイス上にプロットを配置するための
  他の機構と全く一貫性が無い：
  \code{\link{par}(mfrow)},
  \code{par(mfcol)} そして \code{\link{split.screen}}．
}
\examples{
def.par <- par(no.readonly = TRUE) # リセットのために既定値を保存 ...

## デバイスを二つの行と二つの列に分割
## 図 1 を行 1に置く
## 図 2 を列 2 と 行 2 の交際地に置く
layout(matrix(c(1,1,0,2), 2, 2, byrow = TRUE))
## 各プロットに確保された領域を示す
layout.show(2)

## デバイスを二つの行と二つの列に分割
## 図 1 と図 2 を上の様に確保
## 幅と高さの間の関係を尊重
nf <- layout(matrix(c(1,1,0,2), 2, 2, byrow = TRUE), respect = TRUE)
layout.show(nf)

## 5cm 四方の単一の正方形を作る
nf <- layout(matrix(1), widths = lcm(5), heights = lcm(5))
layout.show(nf)

##-- 周辺にヒストグラムを持つ散布図を作る

x <- pmin(3, pmax(-3, stats::rnorm(50)))
y <- pmin(3, pmax(-3, stats::rnorm(50)))
xhist <- hist(x, breaks = seq(-3,3,0.5), plot = FALSE)
yhist <- hist(y, breaks = seq(-3,3,0.5), plot = FALSE)
top <- max(c(xhist$counts, yhist$counts))
xrange <- c(-3, 3)
yrange <- c(-3, 3)
nf <- layout(matrix(c(2,0,1,3),2,2,byrow = TRUE), c(3,1), c(1,3), TRUE)
layout.show(nf)

par(mar = c(3,3,1,1))
plot(x, y, xlim = xrange, ylim = yrange, xlab = "", ylab = "")
par(mar = c(0,3,1,1))
barplot(xhist$counts, axes = FALSE, ylim = c(0, top), space = 0)
par(mar = c(3,0,1,1))
barplot(yhist$counts, axes = FALSE, xlim = c(0, top), space = 0, horiz = TRUE)

par(def.par)  #- 既定値にリセット
}
\keyword{iplot}
\keyword{dplot}
\keyword{environment}

\encoding{UTF8}
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% File src/library/graphics/man/layout.Rd
%% Part of the R package, https://www.R-project.org
%% Copyright 1995-2012 R Core Team
%% Distributed under GPL 2 or later
%
%\name{layout}
%\alias{layout}
%\alias{layout.show}
%\alias{lcm}
%\title{Specifying Complex Plot Arrangements}
%\author{Paul R. Murrell}
%\description{
%  \code{layout} divides the device up into as many rows and columns as
%  there are in matrix \code{mat}, with the column-widths and the
%  row-heights specified in the respective arguments.
%}
%\usage{
%layout(mat, widths = rep.int(1, ncol(mat)),
%       heights = rep.int(1, nrow(mat)), respect = FALSE)
%
%layout.show(n = 1)
%lcm(x)
%}
%\arguments{
%  \item{mat}{a matrix object specifying the location of the next
%    \eqn{N} figures on the output device.  Each value in the matrix must
%    be \code{0} or a positive integer.  If \eqn{N} is the largest
%    positive integer in the matrix, then the integers
%    \eqn{\{1, \dots, N-1\}}{{1, \dots, N-1}} must also appear at least once in the matrix.}
%  \item{widths}{a vector of values for the widths of columns on the
%    device.  Relative widths are specified with numeric values.    Absolute
%    widths (in centimetres) are specified with the \code{lcm()} function (see
%    examples).}
%  \item{heights}{a vector of values for the heights of rows on the device.
%    Relative and absolute heights can be specified, see \code{widths} above.}
%  \item{respect}{either a logical value or a matrix object.  If the
%    latter, then it must have the same dimensions as \code{mat} and
%    each value in the matrix must be either \code{0} or \code{1}.}
%  \item{n}{number of figures to plot.}
%  \item{x}{a dimension to be interpreted as a number of centimetres.}
%}
%\details{
%    Figure \eqn{i} is allocated a region composed from a subset
%    of these rows and columns, based on the rows and columns
%    in which \eqn{i} occurs in \code{mat}.
%
%    The \code{respect} argument controls whether a unit column-width is
%    the same physical measurement on the device as a unit row-height.
%
%    There is a limit (currently 200) for the numbers of rows and columns
%    in the layout, and also for the total number of cells (10007).
%
%    \code{layout.show(n)} plots (part of) the current layout, namely the
%    outlines of the next \code{n} figures.
%
%    \code{lcm} is a trivial function, to be used as \emph{the} interface
%    for specifying absolute dimensions for the \code{widths} and
%    \code{heights} arguments of \code{layout()}.
%}
%\value{
%  \code{layout} returns the number of figures, \eqn{N}, see above.
%}
%\references{
%  Murrell, P. R. (1999) Layouts: A mechanism for arranging plots on a page.
%  \emph{Journal of Computational and Graphical Statistics},
%  \bold{8}, 121--134.
%
%  Chapter 5 of Paul Murrell's Ph.D. thesis.
%
%  Murrell, P. (2005) \emph{R Graphics}. Chapman & Hall/CRC Press.
%}
%\seealso{
%  \code{\link{par}} with arguments \code{mfrow}, \code{mfcol}, or
%  \code{mfg}.
%}
%\section{Warnings}{
%  These functions are totally incompatible with the other mechanisms for
%  arranging plots on a device: \code{\link{par}(mfrow)},
%  \code{par(mfcol)} and \code{\link{split.screen}}.
%}
%\examples{
%def.par <- par(no.readonly = TRUE) # save default, for resetting...
%
%## divide the device into two rows and two columns
%## allocate figure 1 all of row 1
%## allocate figure 2 the intersection of column 2 and row 2
%layout(matrix(c(1,1,0,2), 2, 2, byrow = TRUE))
%## show the regions that have been allocated to each plot
%layout.show(2)
%
%## divide device into two rows and two columns
%## allocate figure 1 and figure 2 as above
%## respect relations between widths and heights
%nf <- layout(matrix(c(1,1,0,2), 2, 2, byrow = TRUE), respect = TRUE)
%layout.show(nf)
%
%## create single figure which is 5cm square
%nf <- layout(matrix(1), widths = lcm(5), heights = lcm(5))
%layout.show(nf)
%
%
%##-- Create a scatterplot with marginal histograms -----
%
%x <- pmin(3, pmax(-3, stats::rnorm(50)))
%y <- pmin(3, pmax(-3, stats::rnorm(50)))
%xhist <- hist(x, breaks = seq(-3,3,0.5), plot = FALSE)
%yhist <- hist(y, breaks = seq(-3,3,0.5), plot = FALSE)
%top <- max(c(xhist$counts, yhist$counts))
%xrange <- c(-3, 3)
%yrange <- c(-3, 3)
%nf <- layout(matrix(c(2,0,1,3),2,2,byrow = TRUE), c(3,1), c(1,3), TRUE)
%layout.show(nf)
%
%par(mar = c(3,3,1,1))
%plot(x, y, xlim = xrange, ylim = yrange, xlab = "", ylab = "")
%par(mar = c(0,3,1,1))
%barplot(xhist$counts, axes = FALSE, ylim = c(0, top), space = 0)
%par(mar = c(3,0,1,1))
%barplot(yhist$counts, axes = FALSE, xlim = c(0, top), space = 0, horiz = TRUE)
%
%par(def.par)  #- reset to default
%}
%\keyword{iplot}
%\keyword{dplot}
%\keyword{environment}
%
%\encoding{UTF8}
%